@@ -1,6 +1,16 @@
 
 # Plots -------------------------------------------------------------------
 
+# keep track of plot options
+# to add bound to contours automatically, we need to recover:
+#  1. the lims used
+#  2. what the sensitivity was
+plot.env <- new.env(parent = emptyenv())
+plot.env$lim <- NULL
+plot.env$lim.y <- NULL
+plot.env$reduce <- NULL
+plot.env$sensitivity.of <- NULL
+plot.env$treatment <- NULL
 
 # generic plot function ---------------------------------------------------
 
@@ -20,7 +30,6 @@
 #' extreme scenarios plots of omitted variable bias as in \code{\link{ovb_extreme_plot}}.
 #' @param ... arguments passed to the plot functions. Check arguments in  \code{\link{ovb_contour_plot}}
 #' and \code{\link{ovb_extreme_plot}}.
-#' @inheritParams ovb_contour_plot
 #' @export
 plot.sensemakr = function(x,
                           type = c("contour", "extreme"),
@@ -29,31 +38,10 @@ plot.sensemakr = function(x,
   type <- match.arg(type)
 
   # Call the dispatch function of interest
-  switch(type,
-         "contour" = dispatch_contour,
-         "extreme" = dispatch_extreme)(x,
-                                       ...)
-}
-
-dispatch_contour = function(x, sensitivity.of = c("estimate", "t-value"), digits = 2, ...) {
-  # Use dispatcher rather than direct call so we can allow modifying call if
-  # necessary
-  sensitivity.of <- match.arg(sensitivity.of)
-  estimate <- x$sensitivity_stats$estimate
-  q <- x$info$q
-  reduce <- x$info$reduce
-  alpha <- x$info$alpha
-  dof <- x$sensitivity_stats$dof
-  thr <- round(ifelse(reduce, estimate*(1 - q), estimate*(1 + q) ), digits = digits)
-  t.thr <- round(abs(qt(alpha/2, df = dof))*sign(x$sensitivity_stats$t_statistic), digits = digits)
-  ovb_contour_plot(x, sensitivity.of = sensitivity.of, estimate.threshold = thr, t.threshold = t.thr,
-                   ...)
-}
-
-dispatch_extreme = function(x, ...) {
-  # Use dispatcher rather than direct call so we can allow modifying call if
-  # necessary
-  ovb_extreme_plot(x, ...)
+  plotter <- switch(type,
+                    "contour" = ovb_contour_plot,
+                    "extreme" = ovb_extreme_plot)
+  plotter(x, ...)
 }
 
 
@@ -72,6 +60,15 @@ ovb_contour_plot.sensemakr <- function(x, sensitivity.of = c("estimate", "t-valu
     bound_label = x$bounds$bound_label
   }
 
+  estimate <- x$sensitivity_stats$estimate
+  q <- x$info$q
+  reduce <- x$info$reduce
+  alpha <- x$info$alpha
+  dof <- x$sensitivity_stats$dof
+  thr <- ifelse(reduce, estimate*(1 - q), estimate*(1 + q) )
+  t.thr <- abs(qt(alpha/2, df = dof - 1))*sign(x$sensitivity_stats$t_statistic)
+  plot.env$treatment <- x$info$treatment
+
   with(x,
        ovb_contour_plot(estimate = sensitivity_stats$estimate,
                         se = sensitivity_stats$se,
@@ -80,11 +77,16 @@ ovb_contour_plot.sensemakr <- function(x, sensitivity.of = c("estimate", "t-valu
                         r2yz.dx = r2yz.dx,
                         bound_label = bound_label,
                         sensitivity.of = sensitivity.of,
+                        reduce = reduce,
+                        estimate.threshold = thr,
+                        t.threshold = t.thr,
                         ...)
 
   )
 }
 
+
+
 #' @export
 ovb_extreme_plot.sensemakr <- function(x, r2yz.dx = c(1, 0.75, 0.5), ...){
 
@@ -95,13 +97,18 @@ ovb_extreme_plot.sensemakr <- function(x, r2yz.dx = c(1, 0.75, 0.5), ...){
     r2dz.x <- x$bounds$r2dz.x
     bound_label = x$bounds$bound_label
   }
-
+  estimate <- x$sensitivity_stats$estimate
+  q <- x$info$q
+  reduce <- x$info$reduce
+  thr <- ifelse(reduce, estimate*(1 - q), estimate*(1 + q) )
   with(x,
        ovb_extreme_plot(estimate = sensitivity_stats$estimate,
                         se = sensitivity_stats$se,
                         dof = sensitivity_stats$dof,
                         r2dz.x = r2dz.x,
                         r2yz.dx = r2yz.dx,
+                        reduce = reduce,
+                        threshold = thr,
                         ...)
 
   )
@@ -124,14 +131,14 @@ ovb_extreme_plot.sensemakr <- function(x, r2yz.dx = c(1, 0.75, 0.5), ...){
 #'  The dotted red line show the chosen critical threshold (for instance, zero): confounders with such strength (or stronger) are sufficient to invalidate the research conclusions.
 #'  All results are exact for single confounders and conservative for multiple/nonlinear confounders.
 #'
-#'  See Cinelli and Hazlett (2018) for details.
+#'  See Cinelli and Hazlett (2020) for details.
 #'
 #' @param ... arguments passed to other methods. First argument should either be an \code{\link{lm}} model with the
 #' outcome regression, a \code{\link{formula}} describing the model along
 #' with the \code{\link{data.frame}} containing the variables of the model,
 #' or a numeric vector with the coefficient estimate.
 #'
-#' @references Cinelli, C. and Hazlett, C. "Making Sense of Sensitivity: Extending Omitted Variable Bias." (2018).
+#' @references Cinelli, C. and Hazlett, C. (2020), "Making Sense of Sensitivity: Extending Omitted Variable Bias." Journal of the Royal Statistical Society, Series B (Statistical Methodology).
 #'
 #' @return
 #' The function returns invisibly the data used for the contour plot (contour grid and bounds).
@@ -163,13 +170,15 @@ ovb_contour_plot = function(...) {
 #' or adjusted t-values (\code{"t-value"})?
 #' @param estimate.threshold critical threshold for the point estimate.
 #' @param t.threshold critical threshold for the t-value.
-#' @param lim sets limit of the plot.
+#' @param lim sets limit for x-axis. If `NULL`, limits are computed automatically.
+#' @param lim.y  sets limit for y-axis. If `NULL`, limits are computed automatically.
 #' @param nlevels number of levels for the contour plot.
 #' @param col.contour color of contour lines.
 #' @param col.thr.line color of threshold contour line.
 #' @param label.text should label texts be plotted? Default is \code{TRUE}.
 #' @param label.bump.x bump on the x coordinate of label text.
 #' @param label.bump.y bump on the y coordinate of label text.
+#' @param round number of digits to show in contours and bound values
 #' @export
 ovb_contour_plot.lm = function(model,
                                treatment,
@@ -178,30 +187,22 @@ ovb_contour_plot.lm = function(model,
                                ky = kd,
                                r2dz.x = NULL,
                                r2yz.dx = r2dz.x,
-                               bound_label = NULL,
+                               bound_label = "manual",
                                sensitivity.of = c("estimate", "t-value"),
                                reduce = TRUE,
                                estimate.threshold = 0,
                                t.threshold = 2,
-                               lim = max(c(0.4,r2dz.x,r2yz.dx)),
-                               nlevels = 20,
+                               nlevels = 10,
                                col.contour = "grey40",
                                col.thr.line = "red",
                                label.text = TRUE,
-                               label.bump.x = 0.02,
-                               label.bump.y = 0.02,
+                               cex.label.text = .7,
+                               round = 3,
                                ...) {
 
 
   check_multipliers(ky = ky, kd = kd)
-  if (lim > 1) {
-    lim <- 1
-    warning("Contour limit larger than 1 was set to 1.")
-  }
-  if (lim < 0) {
-    lim <- 0.4
-    warning("Contour limit less than 0 was set to 0.4.")
-  }
+
 
   sensitivity.of <- match.arg(sensitivity.of)
   # extract model data
@@ -219,7 +220,6 @@ ovb_contour_plot.lm = function(model,
                           r2yz.dx = r2yz.dx,
                           bound_label = bound_label,
                           stringsAsFactors = FALSE)
-    lim <- max(c(lim, r2dz.x, r2yz.dx))
   } else{
     bounds <-  NULL
   }
@@ -234,9 +234,11 @@ ovb_contour_plot.lm = function(model,
                                ky = ky,
                                adjusted_estimates = FALSE)
     bounds <- rbind(bounds, bench_bounds)
-    lim <- max(c(lim, bounds$r2dz.x, bounds$r2yz.dx))
   }
 
+  # update treatment env
+  plot.env$treatment <- treatment
+
   ovb_contour_plot(estimate = estimate,
                    se = se,
                    dof = dof,
@@ -247,13 +249,12 @@ ovb_contour_plot.lm = function(model,
                    bound_label = bounds$bound_label,
                    sensitivity.of = sensitivity.of,
                    t.threshold = t.threshold,
-                   lim = lim,
                    nlevels = nlevels,
                    col.contour = col.contour,
                    col.thr.line = col.thr.line,
                    label.text = label.text,
-                   label.bump.x = label.bump.x,
-                   label.bump.y = label.bump.y,
+                   cex.label.text = cex.label.text,
+                   round = round,
                    ...)
 
 
@@ -276,11 +277,12 @@ ovb_contour_plot.formula = function(formula,
                                     reduce = TRUE,
                                     estimate.threshold = 0,
                                     t.threshold = 2,
-                                    lim = max(c(0.4,r2dz.x,r2yz.dx)),
-                                    nlevels = 20,
+                                    nlevels = 10,
                                     col.contour = "grey40",
                                     col.thr.line = "red",
                                     label.text = TRUE,
+                                    cex.label.text = .7,
+                                    round = 3,
                                     ...) {
 
   check_formula(treatment = treatment,
@@ -308,11 +310,12 @@ ovb_contour_plot.formula = function(formula,
                    bound_label = bound_label,
                    sensitivity.of = sensitivity.of,
                    t.threshold = t.threshold,
-                   lim = lim,
                    nlevels = nlevels,
                    col.contour = col.contour,
                    col.thr.line = col.thr.line,
                    label.text = label.text,
+                   cex.label.text = cex.label.text,
+                   round = round,
                    ...)
 }
 
@@ -322,39 +325,96 @@ ovb_contour_plot.formula = function(formula,
 #' @importFrom graphics contour points text
 #' @importFrom stats coef
 #' @param cex.label.text size of the label text.
+#' @param xlab label of x axis. If `NULL`, default label is used.
+#' @param ylab label of y axis. If `NULL`, default label is used.
+#' @param list.par  arguments to be passed to \code{\link{par}}. It needs to be a named list.
+#' @param cex.lab The magnification to be used for x and y labels relative to the current setting of cex.
+#' @param cex.main The magnification to be used for main titles relative to the current setting of cex.
+#' @param cex.axis The magnification to be used for axis annotation relative to the current setting of cex.
+#' @param asp the y/x aspect ratio. Default is 1.
 #' @export
 ovb_contour_plot.numeric = function(estimate,
                                     se,
                                     dof,
                                     r2dz.x = NULL,
                                     r2yz.dx = r2dz.x,
-                                    bound_label = "",
+                                    bound_label = rep("manual", length(r2dz.x)),
                                     sensitivity.of = c("estimate", "t-value"),
                                     reduce = TRUE,
                                     estimate.threshold = 0,
                                     t.threshold = 2,
-                                    lim = max(c(0.4, r2dz.x + 0.1,r2yz.dx + 0.1)),
-                                    nlevels = 20,
+                                    lim = NULL,
+                                    lim.y = NULL,
+                                    nlevels = 10,
                                     col.contour = "black",
                                     col.thr.line = "red",
                                     label.text = TRUE,
-                                    cex.label.text = 1,
-                                    label.bump.x = 0.02,
-                                    label.bump.y = 0.02,
+                                    cex.label.text = .7,
+                                    label.bump.x = NULL,
+                                    label.bump.y = NULL,
+                                    xlab = NULL,
+                                    ylab = NULL,
+                                    cex.lab = .8,
+                                    cex.axis = .8,
+                                    cex.main = 1,
+                                    asp = lim/lim.y,
+                                    list.par = list(mar = c(4,4,1,1), pty = "s"),
+                                    round = 3,
                                     ...) {
 
   check_estimate(estimate)
   check_r2(r2dz.x = r2dz.x, r2yz.dx = r2yz.dx)
-  lim <- check_contour_lim(lim)
+  if (length(r2dz.x) != length(r2yz.dx)) {
+    stop("Length of r2dz.x and r2yz.dx partial R2 must match")
+  }
+
+  if (is.null(lim)){
+    lim   <- min(max(c(0.4, r2dz.x*1.2)), 1 - 1e-12)
+  }
+
+  if (is.null(lim.y)){
+    lim.y <- min(max(c(0.4, r2yz.dx*1.2)), 1 - 1e-12)
+  }
+
+  if (is.null(label.bump.x)){
+    label.bump.x <- lim*(1/15)
+  }
+
+  if (is.null(label.bump.y)){
+    label.bump.y <- lim.y*(1/15)
+  }
+
+  if (lim > 1) {
+    lim <- 1 - 1e-12
+    warning("Contour limit larger than 1 was set to 1.")
+  }
+
+  if (lim.y > 1) {
+    lim.y <- 1 - 1e-12
+    warning("Contour limit larger than 1 was set to 1.")
+  }
+
+  if (lim < 0) {
+    lim <- 0.4
+    warning("Contour limit less than 0 was set to 0.4.")
+  }
+
+  if (lim.y < 0) {
+    lim.y <- 0.4
+    warning("Contour limit less than 0 was set to 0.4.")
+  }
+
+
 
   sensitivity.of <- match.arg(sensitivity.of)
 
   # Set up the grid for the contour plot
-  grid_values = seq(0, lim, by = lim/400)
+  grid_values.x = seq(0, lim, by = lim/400)
+  grid_values.y = seq(0, lim.y, by = lim.y/400)
 
   # Are we plotting t or bias in r2?
   if (sensitivity.of == "estimate") {
-    z_axis = outer(grid_values, grid_values,
+    z_axis = outer(grid_values.x, grid_values.y,
                    FUN = "adjusted_estimate",
                    estimate = estimate,
                    se = se,
@@ -374,9 +434,11 @@ ovb_contour_plot.numeric = function(estimate,
   }
 
   if (sensitivity.of == "t-value") {
-    z_axis = outer(grid_values, grid_values,
+    z_axis = outer(grid_values.x, grid_values.y,
                    FUN = "adjusted_t",
-                   se = se, dof = dof, estimate = estimate,
+                   se = se, dof = dof,
+                   estimate = estimate,
+                   reduce = reduce,
                    h0 = estimate.threshold) # we are computing the t-value of H0: tau = estimate.threshold
     threshold = t.threshold
     plot_estimate = (estimate - estimate.threshold) / se
@@ -391,40 +453,72 @@ ovb_contour_plot.numeric = function(estimate,
 
   }
 
-  out = list(r2dz.x = grid_values,
-             r2yz.dx = grid_values,
+  out = list(r2dz.x = grid_values.x,
+             r2yz.dx = grid_values.y,
              value = z_axis)
 
   # Aesthetic: Override the 0 line; basically, check which contour curve is
   # the zero curve, and override that contour curve with alternate aesthetic
   # characteristics
   default_levels = pretty(range(z_axis), nlevels)
-  line_color = ifelse(default_levels == threshold,
+
+  too_close <- abs(default_levels - threshold) < min(diff(default_levels))*0.25
+
+  line_color = ifelse(too_close,
                       "transparent",
                       col.contour)
-  line_type = ifelse(default_levels == threshold,
+
+  line_type = ifelse(too_close,
                      1, 1)
-  line_width = ifelse(default_levels == threshold,
+  line_width = ifelse(too_close,
                       1, 1)
 
   # Plot contour plot:
-  oldpar <- par(mar = c(5, 5, 4, 1) + .1)
-  on.exit(par(oldpar))
+  if (is.null(list.par)) {
+    oldpar <- par(mar = c(5, 5, 4, 1) + .1)
+    on.exit(par(oldpar))
+  } else {
+    if (!is.list(list.par)) stop("list.par needs to be a named list")
+    oldpar <- do.call("par", list.par)
+    on.exit(par(oldpar))
+  }
+
+
+  if (is.null(xlab)) {
+    xlab <- expression(paste("Partial ", R^2, " of confounder(s) with the treatment"))
+  }
+
+  if (is.null(ylab)) {
+    ylab <- expression(paste("Partial ", R^2, " of confounder(s) with the outcome"))
+  }
 
   contour(
-    grid_values, grid_values, z_axis, nlevels = nlevels,
-    xlab = expression(paste("Hypothetical partial ", R^2, " of unobserved",
-                            " confounder(s) with the treatment")),
-    ylab = expression(paste("Hypothetical partial ", R^2, " of unobserved",
-                            " confounder(s) with the outcome")),
-    cex.main = 1, cex.lab = 1, cex.axis = 1,
-    col = line_color, lty = line_type, lwd = line_width,
+    grid_values.x, grid_values.y, z_axis,
+    nlevels = nlevels,
+    xlab = xlab,
+    ylab = ylab,
+    cex.main = cex.main,
+    cex.lab = cex.lab,
+    cex.axis = cex.axis,
+    asp = asp,
+    col = line_color,
+    lty = line_type,
+    lwd = line_width,
     ...)
-  contour(grid_values, grid_values,
-          z_axis, level = threshold,
+
+  contour(grid_values.x, grid_values.y,
+          z_axis,
+          level = threshold,
+          label = round(threshold, digits = round),
           add = TRUE,
           col = col.thr.line,
-          lwd = 2, lty = 2, ...)
+          lwd = 2,
+          lty = 2,
+          cex.main = cex.main,
+          cex.lab = cex.lab,
+          cex.axis = cex.axis,
+          asp = asp,
+          ...)
 
   # Add the point of the initial estimate.
   points(0, 0, pch = 17, col = "black", cex = 1)
@@ -448,12 +542,22 @@ ovb_contour_plot.numeric = function(estimate,
                          sensitivity.of = sensitivity.of,
                          label.text = label.text,
                          label.bump.x = label.bump.x,
-                         label.bump.y = label.bump.y)
+                         label.bump.y = label.bump.y,
+                         cex.label.text = cex.label.text,
+                         round = round)
     out$bounds = data.frame(r2dz.x = r2dz.x,
                             r2yz.dx = r2yz.dx,
-                            bound_label = bound_label, stringsAsFactors = FALSE)
+                            bound_label = bound_label,
+                            stringsAsFactors = FALSE)
   }
 
+  # update plot environment variables
+  # for further use with add_bounds_to_contour if needed
+  plot.env$lim <- lim
+  plot.env$lim.y <- lim.y
+  plot.env$reduce <- reduce
+  plot.env$sensitivity.of <- sensitivity.of
+
   invisible(out)
 }
 
@@ -474,17 +578,15 @@ ovb_contour_plot.numeric = function(estimate,
 #'                          pastvoted + hhsize_darfur + female + village,
 #'                          data = darfur)
 #' # contour plot
-#' ovb_contour_plot(model, treatment = "directlyharmed")
+#' ovb_contour_plot(model = model, treatment = "directlyharmed")
 #'
 #' # add bound 3/1 times stronger than female
-#' add_bound_to_contour(model,
-#'                      treatment = "directlyharmed",
+#' add_bound_to_contour(model = model,
 #'                      benchmark_covariates = "female",
 #'                      kd = 3, ky = 1)
 #'
 #' # add bound 50/2 times stronger than age
-#' add_bound_to_contour(model,
-#'                      treatment = "directlyharmed",
+#' add_bound_to_contour(model = model,
 #'                      benchmark_covariates = "age",
 #'                      kd = 50, ky = 2)
 #'
@@ -497,20 +599,70 @@ add_bound_to_contour <- function(...){
   UseMethod("add_bound_to_contour")
 }
 
+
+
 #' @export
 add_bound_to_contour.ovb_bounds <- function(bounds,
-                                            bound_value = NULL,
                                             label.text = TRUE,
-                                            cex.label.text = 1,
-                                            label.bump.x = 0.02,
-                                            label.bump.y = 0.02,
+                                            bound_label = bounds$bound_label,
+                                            bound_value = NULL,
+                                            label.bump.x = plot.env$lim*(1/15),
+                                            label.bump.y = plot.env$lim.y*(1/15),
                                             round = 2,
+                                            cex.label.text = .7,
                                             ...){
+
+
+  if (is.null(plot.env$treatment)) {
+    stop("No treatment found. Please draw a contour plot first, or provide the treatment variable name manually.")
+  }
+
+  # gets bound from environment
+  if (bounds$treatment != plot.env$treatment) {
+  warning("Treament variable of bounds (",  bounds$treatment, ") ",
+          "differs from the treatment variable of the last contour plot (",
+          plot.env$treatment, ").")
+}
+
+  if (is.null(bound_value) & !is.null(plot.env$sensitivity.of)) {
+    if (plot.env$sensitivity.of == "estimate") {
+      bound_value <- bounds$adjusted_estimate
+    } else {
+      bound_value <- bounds$adjusted_t
+    }
+  }
   add_bound_to_contour(r2dz.x = bounds$r2dz.x,
                        r2yz.dx = bounds$r2yz.dx,
                        bound_value = bound_value,
-                       bound_label = bounds$bound_label,
+                       bound_label = bound_label,
                        label.text = label.text,
+                       cex.label.text = cex.label.text,
+                       label.bump.x = label.bump.x,
+                       label.bump.y = label.bump.y,
+                       round = round,
+                       ...)
+}
+
+
+#' @export
+add_bound_to_contour.ovb_partial_r2_bound <- function(bounds,
+                                                      label.text = TRUE,
+                                                      bound_label = bounds$bound_label,
+                                                      bound_value = NULL,
+                                                      label.bump.x = plot.env$lim*(1/15),
+                                                      label.bump.y = plot.env$lim.y*(1/15),
+                                                      round = 2,
+                                                      cex.label.text = .7,
+                                                      ...){
+  # gets bound from environment
+  #
+
+  add_bound_to_contour(r2dz.x = bounds$r2dz.x,
+                       r2yz.dx = bounds$r2yz.dx,
+                       bound_value = bound_value,
+                       bound_label = bound_label,
+                       label.text = label.text,
+                       cex.label.text = cex.label.text,
                        label.bump.x = label.bump.x,
                        label.bump.y = label.bump.y,
                        round = round,
@@ -521,20 +673,32 @@ add_bound_to_contour.ovb_bounds <- function(bounds,
 #' @rdname add_bound_to_contour
 #' @export
 add_bound_to_contour.lm <- function(model,
-                                    treatment,
                                     benchmark_covariates,
                                     kd = 1,
                                     ky = kd,
-                                    reduce = TRUE,
-                                    sensitivity.of = c("estimate", "t-value"),
+                                    bound_label = NULL,
+                                    treatment = plot.env$treatment,
+                                    reduce = plot.env$reduce,
+                                    sensitivity.of = plot.env$sensitivity.of,
                                     label.text = TRUE,
-                                    cex.label.text = 1,
-                                    label.bump.x = 0.02,
-                                    label.bump.y = 0.02,
+                                    cex.label.text = .7,
+                                    label.bump.x = plot.env$lim*(1/15),
+                                    label.bump.y = plot.env$lim.y*(1/15),
                                     round = 2,
                                     ...)
 {
   sensitivity.of <- match.arg(sensitivity.of)
+
+  if (is.null(plot.env$treatment)) {
+    stop("No treatment found. Please draw a contour plot first, or provide the treatment variable name manually.")
+  }
+
+  if (treatment != plot.env$treatment) {
+    warning("Treament variable provided (",  treatment, ") ",
+            "differs from the treatment variable of the last contour plot (",
+            plot.env$treatment, ").")
+  }
+
   # we will need to add an option for the bound type
   bounds <- ovb_bounds(model = model,
                        treatment = treatment,
@@ -551,11 +715,17 @@ add_bound_to_contour.lm <- function(model,
   if (sensitivity.of == "t-value") {
     bound_value <- bounds$adjusted_t
   }
+
+  if (is.null(bound_label)) {
+   bound_label <-  bounds$bound_label
+  }
+
   add_bound_to_contour(r2dz.x = bounds$r2dz.x,
                        r2yz.dx = bounds$r2yz.dx,
                        bound_value = bound_value,
-                       bound_label = bounds$bound_label,
+                       bound_label = bound_label,
                        label.text = label.text,
+                       cex.label.text = cex.label.text,
                        label.bump.x = label.bump.x,
                        label.bump.y = label.bump.y,
                        round = round,
@@ -572,9 +742,9 @@ add_bound_to_contour.numeric <- function(r2dz.x,
                                          bound_value = NULL,
                                          bound_label = NULL,
                                          label.text = TRUE,
-                                         cex.label.text = 1,
-                                         label.bump.x = 0.02,
-                                         label.bump.y = 0.02,
+                                         cex.label.text = .7,
+                                         label.bump.x = plot.env$lim*(1/15),
+                                         label.bump.y = plot.env$lim.y*(1/15),
                                          round = 2,
                                          ...){
 
@@ -619,7 +789,7 @@ add_bound_to_contour.numeric <- function(r2dz.x,
 #' The red marks on horizontal axis are bounds on the partial R2 of the unobserved confounder \code{kd} times as strong as the covariates used for benchmarking.
 #' The dotted red line represent the threshold for the effect estimate deemed to be problematic (for instance, zero).
 #'
-#' See Cinelli and Hazlett (2018) for details.
+#' See Cinelli and Hazlett (2020) for details.
 #'
 #' @examples
 #'
@@ -633,7 +803,7 @@ add_bound_to_contour.numeric <- function(r2dz.x,
 #'                         kd = 1:2,
 #'                         lim = 0.05)
 #'
-#' @references Cinelli, C. and Hazlett, C. "Making Sense of Sensitivity: Extending Omitted Variable Bias." (2018).
+#' @references Cinelli, C. and Hazlett, C. (2020), "Making Sense of Sensitivity: Extending Omitted Variable Bias." Journal of the Royal Statistical Society, Series B (Statistical Methodology).
 #'
 #' @return
 #' The function returns invisibly the data used for the extreme plot.
@@ -662,8 +832,8 @@ ovb_extreme_plot.lm <- function(model,
                                 threshold = 0,
                                 lim = min(c(r2dz.x + 0.1, 0.5)),
                                 legend = TRUE,
-                                cex.legend = 0.5,
-                                legend.bty = "o",
+                                cex.legend = 0.65,
+                                legend.bty = "n",
                                 ...){
 
   # extract model data
@@ -730,8 +900,8 @@ ovb_extreme_plot.formula = function(formula,
                                     threshold = 0,
                                     lim = min(c(r2dz.x + 0.1, 0.5)),
                                     legend = TRUE,
-                                    cex.legend = 0.5,
-                                    legend.bty = "o",
+                                    cex.legend = 0.65,
+                                    legend.bty = "n",
                                     ...) {
   check_formula(treatment = treatment,
                 formula = formula,
@@ -761,6 +931,9 @@ ovb_extreme_plot.formula = function(formula,
 #' @importFrom graphics abline lines legend rug plot par
 #' @rdname ovb_extreme_plot
 #' @param legend.bty legend box. See \code{bty} argument of \link{par}.
+#' @param legend.title the legend title. If \code{NULL}, then default legend is used.
+#' @param xlab label of x axis. If `NULL`, default label is used.
+#' @param ylab label of y axis. If `NULL`, default label is used.
 #' @export
 ovb_extreme_plot.numeric = function(estimate,
                                     se,
@@ -771,8 +944,14 @@ ovb_extreme_plot.numeric = function(estimate,
                                     threshold = 0,
                                     lim = min(c(r2dz.x + 0.1, 0.5)),
                                     legend = TRUE,
-                                    cex.legend = 0.5,
-                                    legend.bty = "o",
+                                    legend.title  = NULL,
+                                    cex.legend = 0.65,
+                                    legend.bty = "n",
+                                    xlab = NULL,
+                                    ylab = NULL,
+                                    cex.lab = .7,
+                                    cex.axis = .7,
+                                    list.par = list(oma = c(1, 1, 1, 1)),
                                     ...) {
 
   # if (is.null(lim)) {
@@ -782,9 +961,9 @@ ovb_extreme_plot.numeric = function(estimate,
   #   }
   #   lim = max(r2dz.x, na.rm = TRUE) + 0.1
   # }
-  if (lim > 1) {
-    lim <- 1
-    warning("Plot limit larger than 1 was set to 1.")
+  if (lim >= 1) {
+    lim <- 0.99
+    warning("Plot limit on the partial R2 of confounder with the treatment was larger than or equal to 1 and thus was set to 0.99.")
   }
   if (lim < 0) {
     lim <- 0.4
@@ -794,12 +973,21 @@ ovb_extreme_plot.numeric = function(estimate,
   # x-axis values: R^2 of confounder(s) with treatment
   r2d_values = seq(0, lim, by = 0.001)
   out = list()
+
+  if (!is.null(list.par)) {
+    if (!is.list(list.par)) stop("list.par needs to be a named list")
+    oldpar <- do.call("par", list.par)
+    on.exit(par(oldpar))
+  }
+
   # Iterate through scenarios:
   for (i in seq.int(length(r2yz.dx))) {
-    y = adjusted_estimate(estimate, r2yz.dx = r2yz.dx[i],
+    y = adjusted_estimate(estimate,
+                          r2yz.dx = r2yz.dx[i],
                           r2dz.x = r2d_values,
                           se = se,
-                          dof = dof, reduce = reduce)
+                          dof = dof,
+                          reduce = reduce)
     # Initial plot
     if (i == 1) {
       if (estimate < 0) {
@@ -807,12 +995,23 @@ ovb_extreme_plot.numeric = function(estimate,
       } else {
         ylim = range(y)
       }
+
+      if (is.null(xlab)) {
+        xlab <- expression(paste("Partial ", R^2, " of confounder(s) with the treatment"))
+
+      }
+
+      if (is.null(ylab)) {
+        ylab <- "Adjusted effect estimate"
+      }
+
       plot(
         r2d_values, y, type = "l", bty = "L",
         ylim = ylim,
-        xlab = expression(paste("Hypothetical partial ", R^2, " of unobserved",
-                                " confounder(s) with the treatment")),
-        ylab = "Adjusted effect estimate",
+        xlab = xlab,
+        ylab = ylab,
+        cex.lab  = cex.lab,
+        cex.axis = cex.axis,
         ...)
       abline(h = threshold, col = "red", lty = 5)
     } else {
@@ -824,28 +1023,41 @@ ovb_extreme_plot.numeric = function(estimate,
                                                                 adjusted_estimate = y)
 
   }
+  if (!is.null(r2dz.x)) {
+    rug(x = r2dz.x, col = "red", lwd = 2)
+    out[["bounds"]] <- r2dz.x
+  }
+
+  if (is.null(legend.title)) {
+    legend.title <- expression(paste("Partial ", R^2, " of confounder(s) with the outcome"))
+  }
+
 
   if (legend) {
+    old.par <- par(fig = c(0, 1, 0, 1),
+                   oma = c(0, 0, 2, 2),
+                   mar = c(0, 0, 1.5, 0),
+                   new = TRUE)
+    on.exit(par(old.par))
+    plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n", xlab = "", ylab = "")
+
     legend(
       x = "topright",
-      inset = 0.05,
+      inset = c(0,0),
       bty = legend.bty,
+      xpd = TRUE,
       lty = c(seq_len(length(r2yz.dx)), 5),
       col = c(rep("black", length(r2yz.dx)),
               "red"),
       # bty = "n",
       legend = paste0(r2yz.dx * 100, "%"),
       ncol = length(r2yz.dx) + 1,
-      title = expression(paste("Hypothetical partial ", R^2, " of unobserved",
-                               " confounder(s) with the outcome")),
+      title = legend.title,
       cex = cex.legend
     )
   }
 
-  if (!is.null(r2dz.x)) {
-    rug(x = r2dz.x, col = "red", lwd = 2)
-    out[["bounds"]] <- r2dz.x
-  }
+
   return(invisible(out))
 }
 
@@ -862,10 +1074,7 @@ check_formula <- function(treatment, formula, data) {
     stop("You must provide a `treatment` variable present in the model ",
          "`formula` and `data` data frame.")
   }
-  if (!is.null(data) && (!is.data.frame(data) || nrow(data) < 1)) {
-    stop("The provided `data` argument must be a data frame with at least ",
-         "one row.")
-  }
+
 
 }
 
@@ -883,24 +1092,16 @@ check_estimate = function(estimate) {
 }
 
 check_r2 = function(r2dz.x, r2yz.dx) {
-  # Check r2dz.x / r2yz.dx
-  if (is.null(r2dz.x) != is.null(r2yz.dx)) {
-    stop("Either both `r2dz.x` and `r2yz.dx` must be provided or neither.")
-  }
 
   if (!is.null(r2dz.x)) {
     if (any(!is.numeric(r2dz.x) | r2dz.x < 0 | r2dz.x > 1)) {
-      stop("`r2dz.x` must be a number between zero and one, if provided")
-    }
-
-    if (length(r2dz.x) != length(r2yz.dx)) {
-      stop("Lengths of `r2dz.x` and `r2yz.dx` must match.")
+      stop("partial R2 must be a number between zero and one")
     }
   }
 
   if (!is.null(r2yz.dx) &&
      any(!is.numeric(r2yz.dx) | r2yz.dx < 0 | r2yz.dx > 1)) {
-    stop("`r2yz.dx` must be a number between zero and one, if provided")
+    stop("partial R2 must be a number between zero and one, if provided")
   }
 }
 
@@ -916,20 +1117,3 @@ check_multipliers = function(ky, kd) {
   }
 }
 
-
-check_contour_lim <- function(lim) {
-
-  if (lim > 1) {
-    lim <- 1
-    warning("Contour limit larger than 1 was set to 1.")
-    return(lim)
-  }
-
-  if (lim < 0) {
-    lim <- 0.4
-    warning("Contour limit less than 0 was set to 0.4.")
-    return(lim)
-  }
-
-  return(lim)
-}
